==> shelf_oauth-0.2.0.tar.gz: lib/src/oauth2.dart <==
// Copyright (c) 2014, The shelf_oauth authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_oauth.oauth2;

import 'package:shelf/shelf.dart';
import 'dart:async';
import 'package:option/option.dart';
import 'oauth2_impl.dart';
import 'package:uri/uri.dart';
import 'package:shelf_oauth/src/preconditions.dart';

class OAuth2State {
  final String state;
  final String sessionIdentifier;

  OAuth2State(this.state, this.sessionIdentifier) {
    ensure(state, isNotNull);
    ensure(sessionIdentifier, isNotNull);
  }
}

class OAuth2TokenPair {
  final String accessToken;
  final Option<String> refreshToken;
  final DateTime expires;

  OAuth2TokenPair(this.accessToken, this.refreshToken, this.expires) {
    ensure(accessToken, isNotNull);
    ensure(refreshToken, isNotNull);
    ensure(expires, isNotNull);
  }

  OAuth2TokenPair.fromJson(Map json) : this(json['accessToken'],
          new Option(json['refreshToken']), DateTime.parse(json['expires']));

  Map toJson() {
    final m = {
      'accessToken': accessToken,
      'expires': expires.toIso8601String()
    };
    return refreshToken
        .map((rt) => m..['refreshToken'] = rt)
        .getOrElse(() => m);
  }
}

/// A store for OAuth2 request 'state' parameters which are used to prevent
/// CSRF attacks on the redirection Uri. State values are only stored for a
/// short time during the oauth2 dance
abstract class OAuth2CSRFStateStore {
  // TODO: need to include oauth provider as part of key as the authToken's
  // will not be guaranteed to be unique across providers. Although probably
  // low odds of getting duped tokens given the are short lived.

  /// stores the secret for the token
  Future storeState(OAuth2State state);

  /// retrieves and removes the secret for the token
  Future<Option<OAuth2State>> consumeState(String state);
}

abstract class OAuth2TokenStore {
  Future storeToken(String sessionIdentifier, OAuth2TokenPair tokenPair);

  Future<Option<OAuth2TokenPair>> fetchToken(String sessionIdentifier);
}

/// An in memory implementation of [OAuth2CSRFStateStore] suitable for
/// testing only
class InMemoryOAuth2CSRFStateStore implements OAuth2CSRFStateStore {
  final Map<String, OAuth2State> tokenMap = {};

  @override
  Future storeState(OAuth2State state) {
    tokenMap[state.state] = state;
    return new Future.value();
  }

  @override
  Future<Option<OAuth2State>> consumeState(String state) {
    return new Future.value(new Option(tokenMap.remove(state)));
  }
}

class InMemoryOAuth2TokenStore implements OAuth2TokenStore {
  final Map<String, OAuth2TokenPair> tokenMap = {};

  @override
  Future storeToken(String sessionIdentifier, OAuth2TokenPair tokenPair) {
    tokenMap[sessionIdentifier] = tokenPair;
    return new Future.value();
  }

  @override
  Future<Option<OAuth2TokenPair>> fetchToken(String sessionIdentifier) {
    return new Future.value(new Option(tokenMap[sessionIdentifier]));
  }
}

UriTemplate oauth2AuthorizationTemplate(Uri baseUri) {
  final template = baseUri.toString() +
      '{?response_type,client_id,state,scope,access_type,redirect_uri}';
  return new UriTemplate(template);
}

/// A provider of OAuth services
class OAuth2Provider {
  final UriTemplate authorizationUrlTemplate;
  final Uri accessTokenUrl;

  const OAuth2Provider(this.authorizationUrlTemplate, this.accessTokenUrl);

  OAuth2Provider.std(Uri baseAuthorizationUrl, Uri accessTokenUrl)
      : this(oauth2AuthorizationTemplate(baseAuthorizationUrl), accessTokenUrl);
  Uri authorizationUrl(String responseType, String clientId, String state,
      List<String> scopes, Uri redirectUri,
      {String accessType: 'online'}) => Uri.parse(authorizationUrlTemplate
          .expand({
    'response_type': responseType,
    'client_id': clientId,
    'state': state,
    'scope': scopes.join(' '),
    'access_type': accessType,
    'redirect_uri': redirectUri.toString()
  }));

  String accessTokenBody(String grantType, String authorizationCode,
      String clientId, String clientSecret, Uri redirectUri) {
    final Map bodyMap = {
      'grant_type': grantType,
      'code': authorizationCode,
      'client_id': clientId,
      'client_secret': clientSecret, // TODO: is this standard?
      'redirect_uri': redirectUri.toString()
    };

    final body = bodyMap.keys
        .map((k) => '$k=${Uri.encodeComponent(bodyMap[k])}')
        .join('&');

    return body;
  }
}

typedef Future<String> SessionIdentifierExtractor(Request request);

/// shelf [Handler]'s for the oauth2 dance
abstract class OAuth2ProviderHandlers {
  factory OAuth2ProviderHandlers(ClientId clientId,
      OAuth2Provider oauthProvider, Uri redirectUrl,
      OAuth2CSRFStateStore stateStore, OAuth2TokenStore tokenStore,
      UriTemplate completionRedirectUrl,
      SessionIdentifierExtractor sessionIdExtractor, List<String> scopes) {
    return new OAuth2ProviderHandlersImpl(clientId, oauthProvider, redirectUrl,
        stateStore, tokenStore, completionRedirectUrl, sessionIdExtractor,
        scopes);
  }

  Handler authorizationRequestHandler();

  Handler accessTokenRequestHandler();
}

/// Represents the client application's credentials.
class ClientId {
  /// The identifier used to identify this application to the server.
  final String identifier;

  /// The client secret used to identify this application to the server.
  final String secret;

  ClientId(this.identifier, this.secret) {
    if (identifier == null) {
      throw new ArgumentError('Argument identifier may not be null.');
    }
  }
}

class Outh2ResponseType {
  static const String code = 'code';
}

class Outh2GrantType {
  static const String authorizationCode = 'authorization_code';
}

==> shelf_oauth-0.6.1.tar.gz: lib/src/oauth2.dart <==
// Copyright (c) 2014, The shelf_oauth authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf_oauth.oauth2;

import 'package:shelf/shelf.dart';
import 'dart:async';
import 'package:option/option.dart';
import 'oauth2_impl.dart';
import 'package:uri/uri.dart';
import 'package:shelf_oauth/src/preconditions.dart';

typedef ClientId ClientIdFactory(Request r);
typedef OAuth2Provider OAuth2ProviderFactory(Request r);

class OAuth2State {
  final String state;
  final String sessionIdentifier;
  final Option<String> context;

  OAuth2State(this.state, this.sessionIdentifier, this.context) {
    ensure(state, isNotNull);
    ensure(sessionIdentifier, isNotNull);
    ensure(context, isNotNull);
  }

  OAuth2State.fromJson(Map json) : this(json['state'],
          json['sessionIdentifier'], new Option(json['context']));

  Map toJson() {
    final m = {'state': state, 'sessionIdentifier': sessionIdentifier};
    _add(m, 'context', context);
    return m;
  }
}

class OAuth2TokenPair {
  final String accessToken;
  final Option<String> refreshToken;
  final Option<DateTime> expires;
  /// if None then assume all requested scopes were granted
  final Option<List<String>> grantedScopes;

  OAuth2TokenPair(
      this.accessToken, this.refreshToken, this.expires, this.grantedScopes) {
    ensure(accessToken, isNotNull);
    ensure(refreshToken, isNotNull);
    ensure(expires, isNotNull);
    ensure(grantedScopes, isNotNull);
  }

  OAuth2TokenPair.fromJson(Map json) : this(json['accessToken'],
          new Option(json['refreshToken']),
          new Option(json['expires']).map((exp) => DateTime.parse(exp)),
          new Option(json['grantedScopes']));

  Map toJson() {
    final m = {'accessToken': accessToken};
    _add(m, 'refreshToken', refreshToken);
    _add(m, 'expires', expires.map((exp) => exp.toIso8601String()));
    _add(m, 'grantedScopes',
        grantedScopes.map((scope) => scope.toIso8601String()));
    return m;
  }
}

void _add(Map map, String key, Option value) {
  if (value is Some) {
    map[key] = value.get();
  }
}

/// A store for OAuth2 request 'state' parameters which are used to prevent
/// CSRF attacks on the redirection Uri. State values are only stored for a
/// short time during the oauth2 dance
abstract class OAuth2CSRFStateStore {
  // TODO: need to include oauth provider as part of key as the authToken's
  // will not be guaranteed to be unique across providers. Although probably
  // low odds of getting duped tokens given they are short lived.

  /// stores the secret for the token
  Future storeState(OAuth2State state);

  /// retrieves and removes the secret for the token
  Future<Option<OAuth2State>> consumeState(String state);
}

abstract class OAuth2TokenStore {
  Future storeToken(String sessionIdentifier, OAuth2TokenPair tokenPair);

  Future<Option<OAuth2TokenPair>> fetchToken(String sessionIdentifier);
}

/// An in memory implementation of [OAuth2CSRFStateStore] suitable for
/// testing only
class InMemoryOAuth2CSRFStateStore implements OAuth2CSRFStateStore {
  final Map<String, OAuth2State> tokenMap = {};

  @override
  Future storeState(OAuth2State state) {
    tokenMap[state.state] = state;
    return new Future.value();
  }

  @override
  Future<Option<OAuth2State>> consumeState(String state) {
    return new Future.value(new Option(tokenMap.remove(state)));
  }
}

class InMemoryOAuth2TokenStore implements OAuth2TokenStore {
  final Map<String, OAuth2TokenPair> tokenMap = {};

  @override
  Future storeToken(String sessionIdentifier, OAuth2TokenPair tokenPair) {
    tokenMap[sessionIdentifier] = tokenPair;
    return new Future.value();
  }

  @override
  Future<Option<OAuth2TokenPair>> fetchToken(String sessionIdentifier) {
    return new Future.value(new Option(tokenMap[sessionIdentifier]));
  }
}

UriTemplate oauth2AuthorizationTemplate(Uri baseUri) {
  final template = baseUri.toString() +
      '{?response_type,client_id,state,scope,access_type,redirect_uri}';
  return new UriTemplate(template);
}

/// A provider of OAuth services
class OAuth2Provider {
  final UriTemplate authorizationUrlTemplate;
  final Uri accessTokenUrl;

  const OAuth2Provider(this.authorizationUrlTemplate, this.accessTokenUrl);

  OAuth2Provider.std(Uri baseAuthorizationUrl, Uri accessTokenUrl)
      : this(oauth2AuthorizationTemplate(baseAuthorizationUrl), accessTokenUrl);
  Uri authorizationUrl(String responseType, String clientId, String state,
      List<String> scopes, Uri redirectUri,
      {String accessType: 'online'}) => Uri.parse(authorizationUrlTemplate
          .expand({
    'response_type': responseType,
    'client_id': clientId,
    'state': state,
    'scope': scopes.join(' '),
    'access_type': accessType,
    'redirect_uri': redirectUri.toString()
  }));

  String accessTokenBody(String grantType, String authorizationCode,
      String clientId, String clientSecret, Uri redirectUri) {
    final Map bodyMap = {
      'grant_type': grantType,
      'code': authorizationCode,
      'client_id': clientId,
      'client_secret': clientSecret, // TODO: is this standard?
      'redirect_uri': redirectUri.toString()
    };

    final body = bodyMap.keys
        .map((k) => '$k=${Uri.encodeComponent(bodyMap[k])}')
        .join('&');

    return body;
  }
}

typedef Future<String> SessionIdentifierExtractor(Request request);

/// shelf [Handler]'s for the oauth2 dance
abstract class OAuth2ProviderHandlers {
  factory OAuth2ProviderHandlers(ClientIdFactory clientIdFactory,
      OAuth2ProviderFactory oauthProviderFactory, Uri redirectUrl,
      OAuth2CSRFStateStore stateStore, OAuth2TokenStore tokenStore,
      UriTemplate completionRedirectUrl,
      SessionIdentifierExtractor sessionIdExtractor, List<String> scopes,
      {bool storeTokens: true, String accessType: 'online'}) {
    return new OAuth2ProviderHandlersImpl(clientIdFactory, oauthProviderFactory,
        redirectUrl, stateStore, tokenStore, completionRedirectUrl,
        sessionIdExtractor, scopes, storeTokens, accessType);
  }

  Handler authorizationRequestHandler();

  Handler accessTokenRequestHandler();
}

/// Represents the client application's credentials.
class ClientId {
  /// The identifier used to identify this application to the server.
  final String identifier;

  /// The client secret used to identify this application to the server.
  final String secret;

  ClientId(this.identifier, this.secret) {
    if (identifier == null) {
      throw new ArgumentError('Argument identifier may not be null.');
    }
  }
}

class Outh2ResponseType {
  static const String code = 'code';
}

class Outh2GrantType {
  static const String authorizationCode = 'authorization_code';
}
